package dev.adriankuta.flights.ui.home

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import dev.adriankuta.flights.domain.types.AirportInfo
import dev.adriankuta.flights.domain.types.City
import dev.adriankuta.flights.domain.types.Coordinates
import dev.adriankuta.flights.domain.types.Country
import dev.adriankuta.flights.domain.types.MacCity
import dev.adriankuta.flights.domain.types.Region
import dev.adriankuta.flights.ui.designsystem.theme.FlightsTheme
import dev.adriankuta.flights.ui.designsystem.theme.PreviewDevices
import dev.adriankuta.flights.ui.home.components.SearchForm
import java.time.LocalDate

@Composable
internal fun HomeScreen(
    viewModel: HomeScreenViewModel = hiltViewModel(),
) {
    val homeUiState by viewModel.uiState.collectAsStateWithLifecycle()

    HomeScreen(
        uiState = homeUiState,
        onOriginAirportSelect = viewModel::selectOriginAirport,
        onDestinationAirportSelect = viewModel::selectDestinationAirport,
        onDateSelect = viewModel::selectDate,
        onAdultCountChange = viewModel::updateAdultCount,
        onTeenCountChange = viewModel::updateTeenCount,
        onChildCountChange = viewModel::updateChildCount,
        onSearch = viewModel::search,
    )
}

@Composable
private fun HomeScreen(
    uiState: HomeUiState,
    onOriginAirportSelect: (AirportInfo) -> Unit,
    onDestinationAirportSelect: (AirportInfo) -> Unit,
    onDateSelect: (LocalDate) -> Unit,
    onAdultCountChange: (Int) -> Unit,
    onTeenCountChange: (Int) -> Unit,
    onChildCountChange: (Int) -> Unit,
    onSearch: () -> Unit,
    modifier: Modifier = Modifier,
) {
    val scrollState = rememberScrollState()
    Box(
        modifier = modifier
            .padding(16.dp)
            .verticalScroll(
                state = scrollState,
            ),
    ) {
        when (uiState) {
            is HomeUiState.Error -> Text("Error")
            HomeUiState.Loading -> Text("Loading")
            is HomeUiState.Success -> SearchForm(
                uiState = uiState,
                onOriginAirportSelect = onOriginAirportSelect,
                onDestinationAirportSelect = onDestinationAirportSelect,
                onDateSelect = onDateSelect,
                onAdultCountChange = onAdultCountChange,
                onTeenCountChange = onTeenCountChange,
                onChildCountChange = onChildCountChange,
                onSearch = onSearch,
            )
        }
    }
}

@PreviewDevices
@Composable
private fun HomeScreenLoadingPreview() {
    FlightsTheme {
        HomeScreen(
            uiState = HomeUiState.Loading,
            onOriginAirportSelect = {},
            onDestinationAirportSelect = {},
            onDateSelect = {},
            onAdultCountChange = {},
            onTeenCountChange = {},
            onChildCountChange = {},
            onSearch = {},
        )
    }
}

@PreviewDevices
@Composable
private fun HomeScreenSuccessPreview() {
    FlightsTheme {
        val mockAirports = listOf(
            AirportInfo(
                code = "WAW",
                name = "Warsaw Chopin Airport",
                seoName = "warsaw",
                isBase = true,
                timeZone = "Europe/Warsaw",
                city = City("WAW", "Warsaw"),
                macCity = MacCity("WARSAW"),
                region = Region("WARSAW_PL", "Warsaw"),
                country = Country("PL", "Poland", "PLN"),
                coordinates = Coordinates(52.1657, 20.9671),
            ),
            AirportInfo(
                code = "KRK",
                name = "Krakow Airport",
                seoName = "krakow",
                isBase = true,
                timeZone = "Europe/Warsaw",
                city = City("KRK", "Krakow"),
                macCity = MacCity("KRAKOW"),
                region = Region("KRAKOW_PL", "Krakow"),
                country = Country("PL", "Poland", "PLN"),
                coordinates = Coordinates(50.0777, 19.7848),
            ),
        )

        HomeScreen(
            uiState = HomeUiState.Success(
                originAirports = mockAirports,
                destinationAirports = mockAirports,
                selectedOriginAirport = mockAirports.first(),
                selectedDestinationAirport = mockAirports.last(),
                selectedDate = LocalDate.now(),
                passengers = PassengersState(
                    adultCount = 2,
                    teenCount = 1,
                    childCount = 1,
                ),
            ),
            onOriginAirportSelect = {},
            onDestinationAirportSelect = {},
            onDateSelect = {},
            onAdultCountChange = {},
            onTeenCountChange = {},
            onChildCountChange = {},
            onSearch = {},
        )
    }
}
