package dev.adriankuta.flights.ui.home

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dev.adriankuta.flights.core.util.Result
import dev.adriankuta.flights.core.util.asResult
import dev.adriankuta.flights.domain.search.ObserveAirportsUseCase
import dev.adriankuta.flights.domain.types.AirportInfo
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.stateIn
import timber.log.Timber
import java.time.LocalDate
import javax.inject.Inject

@HiltViewModel
class HomeScreenViewModel @Inject constructor(
    observeAirportsUseCase: ObserveAirportsUseCase,
) : ViewModel() {

    private val selectedOriginAirport = MutableStateFlow<AirportInfo?>(null)
    private val selectedDestinationAirport = MutableStateFlow<AirportInfo?>(null)
    private val selectedDate = MutableStateFlow(LocalDate.now())
    private val adultCount = MutableStateFlow(1)
    private val teenCount = MutableStateFlow(0)
    private val childCount = MutableStateFlow(0)

    internal val uiState = homeUiState(
        useCase = observeAirportsUseCase,
        selectedOriginAirport = selectedOriginAirport,
        selectedDestinationAirport = selectedDestinationAirport,
        selectedDate = selectedDate,
        passengersState = passengersState(
            adultCount = adultCount,
            teenCount = teenCount,
            childCount = childCount,
        ),
    )
        .stateIn(
            scope = viewModelScope,
            started = SharingStarted.WhileSubscribed(5_000),
            initialValue = HomeUiState.Loading,
        )

    fun selectOriginAirport(airport: AirportInfo) {
        selectedOriginAirport.value = airport
        // If the selected destination is the same as the new origin, clear the destination
        if (selectedDestinationAirport.value?.code == airport.code) {
            selectedDestinationAirport.value = null
        }
    }

    fun selectDestinationAirport(airport: AirportInfo) {
        selectedDestinationAirport.value = airport
    }

    fun clearOriginAirport() {
        selectedOriginAirport.value = null
        // Clear destination as well since it depends on origin
        selectedDestinationAirport.value = null
    }

    fun clearDestinationAirport() {
        selectedDestinationAirport.value = null
    }

    fun selectDate(date: LocalDate) {
        selectedDate.value = date
    }

    fun updateAdultCount(count: Int) {
        adultCount.value = count
        childCount.value = childCount.value.coerceAtMost(count)
    }

    fun updateTeenCount(count: Int) {
        teenCount.value = count
    }

    fun updateChildCount(count: Int) {
        childCount.value = count
    }

    fun search() {
        Timber.d("Search clicked")
    }
}

private fun homeUiState(
    useCase: ObserveAirportsUseCase,
    selectedOriginAirport: StateFlow<AirportInfo?>,
    selectedDestinationAirport: StateFlow<AirportInfo?>,
    selectedDate: StateFlow<LocalDate>,
    passengersState: Flow<PassengersState>,
): Flow<HomeUiState> {
    return combine(
        useCase().asResult(),
        selectedOriginAirport,
        selectedDestinationAirport,
        selectedDate,
        passengersState,
    ) { result, origin, destination, date, passengers ->
        when (result) {
            is Result.Error -> HomeUiState.Error(result.exception)
            is Result.Loading -> HomeUiState.Loading
            is Result.Success -> {
                val airports = result.data.orEmpty()
                HomeUiState.Success(
                    originAirports = airports,
                    destinationAirports = airports.filter { it.code != origin?.code },
                    selectedOriginAirport = origin,
                    selectedDestinationAirport = destination,
                    selectedDate = date,
                    passengers = passengers,
                )
            }
        }
    }
}

private fun passengersState(
    adultCount: StateFlow<Int>,
    teenCount: StateFlow<Int>,
    childCount: StateFlow<Int>,
): Flow<PassengersState> {
    return combine(
        adultCount,
        teenCount,
        childCount,
    ) { adults, teens, children ->
        PassengersState(
            adultCount = adults,
            teenCount = teens,
            childCount = children,
        )
    }
}

internal sealed interface HomeUiState {
    data object Loading : HomeUiState
    data class Success(
        val originAirports: List<AirportInfo>,
        val destinationAirports: List<AirportInfo>,
        val selectedOriginAirport: AirportInfo? = null,
        val selectedDestinationAirport: AirportInfo? = null,
        val selectedDate: LocalDate = LocalDate.now(),
        val passengers: PassengersState,
    ) : HomeUiState

    data class Error(val exception: Throwable) : HomeUiState
}

internal data class PassengersState(
    val adultCount: Int,
    val teenCount: Int,
    val childCount: Int,
)
